<?php

namespace Drupal\menu_family;

use Drupal\node\Entity\Node;
use Drupal\menu_link_content\Plugin\Menu\MenuLinkContent;

/**
 * Abstract class to find related nodes.
 */
abstract class MenuFamilyRelated {

  /**
   * Get the starting node from which getting the related nodes.
   *
   * If the starting node is not found, an empty string is returned.
   *
   * @param string $menu_name
   *   The menu name.
   * @param int $nid
   *   The node ID.
   *
   * @return string
   *   The starting node UUID.
   */
  abstract protected static function getStartingNode(
    string $menu_name,
    int $nid,
  ): string;

  /**
   * Generate related nodes.
   *
   * @param int $nid
   *   The node ID.
   * @param string $display_mode
   *   The display mode.
   * @param string $menu_name
   *   The menu name.
   *
   * @return array
   *   The menu link content data.
   */
  public static function generateNodes(
    int $nid,
    string $display_mode,
    string $menu_name,
  ): array {
    $menu_link_content_uuid = static::getStartingNode($menu_name, $nid);

    if (empty($menu_link_content_uuid)) {
      return [];
    }

    $menu_child_results = static::getRelateds(
      $menu_name,
      static::getStartingNode($menu_name, $nid)
    );

    $nodes_child = [];
    foreach ($menu_child_results as $key => $item) {
      $link = \Drupal::entityTypeManager()
        ->getStorage('menu_link_content')
        ->load($key)
        ->getUrlObject();

      if ($link->getRouteName() !== 'entity.node.canonical') {
        continue;
      }

      $params = $link->getRouteParameters();
      $node_id = $params['node'];

      $nodes_child[] = static::generateViewMode($node_id, $display_mode);
    }

    return $nodes_child;
  }

  /**
   * Find the UUID of the first menu link content of a node.
   *
   * @param string $menu_name
   *   The menu name.
   * @param int $nid
   *   The node ID.
   *
   * @return \Drupal\menu_link_content\Entity\MenuLinkContent|null
   *   The first menu link content found.
   */
  protected static function findNodeMenuLinkContent(
    string $menu_name,
    int $nid,
  ): ?MenuLinkContent {
    $entries = \Drupal::service('plugin.manager.menu.link')
      ->loadLinksByRoute('entity.node.canonical', ['node' => $nid], $menu_name);

    if (empty($entries)) {
      return NULL;
    }

    return reset($entries);
  }

  /**
   * Render view mode for node.
   *
   * @param int $nid
   *   The node object.
   * @param string $viewMode
   *   The view mode machine name.
   *
   * @return array
   *   The HTML view mode.
   */
  protected static function generateViewMode($nid, $viewMode) {
    $node = Node::load($nid);

    $build = \Drupal::entityTypeManager()
      ->getViewBuilder('node')
      ->view($node, $viewMode);

    return [
      '#markup' => \Drupal::service('renderer')->renderRoot($build),
    ];
  }

  /**
   * Get children menu entries for a parent menu link content sorted by weight.
   *
   * @param string $menu_name
   *   The menu name.
   * @param string $parent_menu_link_content_uuid
   *   The parent menu link content UUID.
   *
   * @return array
   *   The children menu entries.
   */
  protected static function getRelateds(
    string $menu_name,
    string $parent_menu_link_content_uuid,
  ): array {
    return \Drupal::entityTypeManager()
      ->getStorage('menu_link_content')
      ->getQuery()
      ->accessCheck(TRUE)
      ->condition('menu_name', $menu_name)
      ->condition('enabled', 1)
      ->condition('parent', $parent_menu_link_content_uuid)
      ->sort('weight', 'ASC')
      ->execute();
  }

  /**
   * Get the node ID to which an entity is finally attached.
   *
   * @param string $entity_id
   *   The entity ID.
   *
   * @return string
   *   The node ID.
   */
  public static function getNodeOfEntity(string $entity_id): string {
    $entity = \Drupal::entityTypeManager()
      ->getStorage('paragraph')
      ->load($entity_id);

    return $entity->get('parent_id')->value;
  }

}
